@@ -8,6 +8,7 @@
 #include <stdlib.h>
 #include <limits.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <zlib.h>
 #include <sys/socket.h>
 #include <cbtcommon/debug.h>
@@ -26,26 +27,26 @@ struct _CvsServerCtx
     int write_fd;
     char root[PATH_MAX];
 
-    int is_pserver;
+    bool is_pserver;
 
     /* buffered reads from descriptor */
     char read_buff[RD_BUFF_SIZE];
     char * head;
     char * tail;
 
-    int compressed;
+    bool compressed;
     z_stream zout;
     z_stream zin;
 
     /* when reading compressed data, the compressed data buffer */
-    char zread_buff[RD_BUFF_SIZE];
+    unsigned char zread_buff[RD_BUFF_SIZE];
 };
 
-static void get_cvspass(char *, const char *);
+static void get_cvspass(char *, const char *, int len);
 static void send_string(CvsServerCtx *, const char *, ...);
 static int read_response(CvsServerCtx *, const char *);
 static void ctx_to_fp(CvsServerCtx * ctx, FILE * fp);
-static int read_line(CvsServerCtx * ctx, char * p);
+static int read_line(CvsServerCtx * ctx, char * p, int len);
 
 static CvsServerCtx * open_ctx_pserver(CvsServerCtx *, const char *);
 static CvsServerCtx * open_ctx_forked(CvsServerCtx *, const char *);
@@ -61,8 +62,8 @@ CvsServerCtx * open_cvs_server(char * p_root, int compress)
 
     ctx->head = ctx->tail = ctx->read_buff;
     ctx->read_fd = ctx->write_fd = -1;
-    ctx->compressed = 0;
-    ctx->is_pserver = 0;
+    ctx->compressed = false;
+    ctx->is_pserver = false;
 
     if (compress)
     {
@@ -90,7 +91,7 @@ CvsServerCtx * open_cvs_server(char * p_root, int compress)
 	}
     }
 
-    strcpy(root, p_root);
+    strcpy_a(root, p_root, PATH_MAX);
 
     tok = strsep(&p, ":");
 
@@ -131,7 +132,7 @@ CvsServerCtx * open_cvs_server(char * p_root, int compress)
 	send_string(ctx, "valid-requests\n");
 
 	/* check for the commands we will issue */
-	read_line(ctx, buff);
+	read_line(ctx, buff, BUFSIZ);
 	if (strncmp(buff, "Valid-requests", 14) != 0)
 	{
 	    debug(DEBUG_APPERROR, "cvs_direct: bad response to valid-requests command");
@@ -141,7 +142,6 @@ CvsServerCtx * open_cvs_server(char * p_root, int compress)
 
 	if (!strstr(buff, " version") ||
 	    !strstr(buff, " rlog") ||
-	    !strstr(buff, " rdiff") || 
 	    !strstr(buff, " diff") ||
 	    !strstr(buff, " co"))
 	{
@@ -150,7 +150,7 @@ CvsServerCtx * open_cvs_server(char * p_root, int compress)
 	    return NULL;
 	}
 	
-	read_line(ctx, buff);
+	read_line(ctx, buff, BUFSIZ);
 	if (strcmp(buff, "ok") != 0)
 	{
 	    debug(DEBUG_APPERROR, "cvs_direct: bad ok trailer to valid-requests command");
@@ -164,10 +164,10 @@ CvsServerCtx * open_cvs_server(char * p_root, int compress)
 	if (compress)
 	{
 	    send_string(ctx, "Gzip-stream %d\n", compress);
-	    ctx->compressed = 1;
+	    ctx->compressed = true;
 	}
 
-	debug(DEBUG_APPMSG1, "cvs_direct initialized to CVSROOT %s", ctx->root);
+	debug(DEBUG_APPMSG2, "cvs_direct: initialized to CVSROOT %s", ctx->root);
     }
 
     return ctx;
@@ -183,47 +183,47 @@ static CvsServerCtx * open_ctx_pserver(CvsServerCtx * ctx, const char * p_root)
     char pass[BUFSIZ];
     char port[8];
 
-    strcpy(root, p_root);
+    strcpy_a(root, p_root, PATH_MAX);
 
-    tok = strsep(&p, ":");
-    if (strlen(tok) == 0 || !p)
-    {
-	debug(DEBUG_APPERROR, "parse error on third token");
-	goto out_free_err;
-    }
-
-    tok2 = strsep(&tok, "@");
-    if (!strlen(tok2) || (!tok || !strlen(tok)))
+    /* parse initial "user@server" portion of p. */
+    tok = strsep(&p, "@");
+    tok2 = p;
+    p += strcspn(p, ":/"); /* server part ends at first ':' or '/'. */
+    if (!tok || !tok2 || !strlen(tok) || 0 >= (p - tok2))
     {
 	debug(DEBUG_APPERROR, "parse error on user@server in pserver");
 	goto out_free_err;
     }
 
-    strcpy(user, tok2);
-    strcpy(server, tok);
-    
-    if (*p != '/')
-    {
-	tok = strchr(p, '/');
-	if (!tok)
-	{
-	    debug(DEBUG_APPERROR, "parse error: expecting / in root");
-	    goto out_free_err;
-	}
-	
-	memset(port, 0, sizeof(port));
-	memcpy(port, p, tok - p);
+    strcpy(user, tok);
+    memcpy(server, tok2, p - tok2);
+    server[p - tok2] = '\0';
 
-	p = tok;
+    /* p now points to ':' or '/' following server part. */
+    tok = strchr(p, '/'); /* find start of path */
+    if (!tok)
+    {
+	debug(DEBUG_APPERROR, "parse error: expecting / in root");
+	goto out_free_err;
+    }
+
+    if (*p == ':') /* port number specified. Ends at tok. */
+    {
+	p++;
+	memcpy(port, p, tok - p);
+	port[tok - p] = '\0';
     }
     else
     {
 	strcpy(port, "2401");
     }
 
+    /* Make p point to path component, starting with '/'. */
+    p = tok;
+
     /* the line from .cvspass is fully qualified, so rebuild */
     snprintf(full_root, PATH_MAX, ":pserver:%s@%s:%s%s", user, server, port, p);
-    get_cvspass(pass, full_root);
+    get_cvspass(pass, full_root, BUFSIZ);
 
     debug(DEBUG_TCP, "user:%s server:%s port:%s pass:%s full_root:%s", user, server, port, pass, full_root);
 
@@ -231,6 +231,8 @@ static CvsServerCtx * open_ctx_pserver(CvsServerCtx * ctx, const char * p_root)
 	goto out_free_err;
 
     ctx->write_fd = dup(ctx->read_fd);
+    if (ctx->write_fd < 0)
+	goto out_close_err;
 
     if (tcp_connect(ctx->read_fd, server, atoi(port)) < 0)
 	goto out_close_err;
@@ -244,8 +246,8 @@ static CvsServerCtx * open_ctx_pserver(CvsServerCtx * ctx, const char * p_root)
     if (!read_response(ctx, "I LOVE YOU"))
 	goto out_close_err;
 
-    strcpy(ctx->root, p);
-    ctx->is_pserver = 1;
+    strcpy_a(ctx->root, p, PATH_MAX);
+    ctx->is_pserver = true;
 
     return ctx;
 
@@ -269,13 +271,14 @@ static CvsServerCtx * open_ctx_forked(CvsServerCtx * ctx, const char * p_root)
     if (!cvs_server)
 	cvs_server = "cvs";
 
-    strcpy(root, p_root);
+    strcpy_a(root, p_root, PATH_MAX);
 
     /* if there's a ':', it's remote */
     tok = strsep(&p, ":");
 
     if (p)
     {
+	/* coverity[tainted_data] */
 	const char * cvs_rsh = getenv("CVS_RSH");
 
 	if (!cvs_rsh)
@@ -327,9 +330,15 @@ static CvsServerCtx * open_ctx_forked(CvsServerCtx * ctx, const char * p_root)
 	close(from_cvs[0]);
 	
 	close(0);
-	dup(to_cvs[0]);
+	if (dup(to_cvs[0]) < 0) {
+	    debug(DEBUG_APPERROR, "cvs_direct: dup of input failed");
+	    exit(1);
+	}
 	close(1);
-	dup(from_cvs[1]);
+	if (dup(from_cvs[1]) < 0) {
+	    debug(DEBUG_APPERROR, "cvs_direct: dup of output failed");
+	    exit(1);
+	}
 
 	execv("/bin/sh",argp);
 
@@ -342,7 +351,7 @@ static CvsServerCtx * open_ctx_forked(CvsServerCtx * ctx, const char * p_root)
     ctx->read_fd = from_cvs[0];
     ctx->write_fd = to_cvs[1];
 
-    strcpy(ctx->root, rep);
+    strcpy_a(ctx->root, rep, PATH_MAX);
 
     return ctx;
 
@@ -373,7 +382,7 @@ void close_cvs_server(CvsServerCtx * ctx)
 	 */
 	do
 	{
-	    ctx->zout.next_out = buff;
+	    ctx->zout.next_out = (unsigned char *)buff;
 	    ctx->zout.avail_out = BUFSIZ;
 	    ret = deflate(&ctx->zout, Z_FINISH);
 
@@ -411,7 +420,7 @@ void close_cvs_server(CvsServerCtx * ctx)
     if (ctx->compressed)
     {
 	int ret = Z_OK, len, eof = 0;
-	char buff[BUFSIZ];
+	unsigned char buff[BUFSIZ];
 
 	/* read to the 'eof'/'eos' marker.  there are two states we 
 	 * track, looking for Z_STREAM_END (application level EOS)
@@ -469,7 +478,7 @@ void close_cvs_server(CvsServerCtx * ctx)
 		debug(DEBUG_TCP, "cvs_direct: got Z_STREAM_END");
 
 	    if ((ret == Z_OK || ret == Z_STREAM_END) && len > 0)
-		hexdump(buff, BUFSIZ - ctx->zin.avail_out, "cvs_direct: zin: unread data at close");
+		hexdump((char *)buff, BUFSIZ - ctx->zin.avail_out, "cvs_direct: zin: unread data at close");
 	}
 
 	if (ret != Z_STREAM_END)
@@ -489,7 +498,7 @@ void close_cvs_server(CvsServerCtx * ctx)
     free(ctx);
 }
 
-static void get_cvspass(char * pass, const char * root)
+static void get_cvspass(char * pass, const char * root, int passbuflen)
 {
     char cvspass[PATH_MAX];
     const char * home;
@@ -522,7 +531,7 @@ static void get_cvspass(char * pass, const char * root)
 
 	    if (strncmp(buff + 3, root, len) == 0)
 	    {
-		strcpy(pass, buff + 3 + len + 1);
+		strcpy_a(pass, buff + 3 + len + 1, passbuflen);
 		chop(pass);
 		break;
 	    }
@@ -538,12 +547,13 @@ static void get_cvspass(char * pass, const char * root)
 static void send_string(CvsServerCtx * ctx, const char * str, ...)
 {
     int len;
-    char buff[BUFSIZ];
+    unsigned char buff[BUFSIZ];
     va_list ap;
 
     va_start(ap, str);
+    len = vsnprintf((char *)buff, BUFSIZ, str, ap);
+    va_end(ap);
 
-    len = vsnprintf(buff, BUFSIZ, str, ap);
     if (len >= BUFSIZ)
     {
 	debug(DEBUG_APPERROR, "cvs_direct: command send string overflow");
@@ -552,7 +562,7 @@ static void send_string(CvsServerCtx * ctx, const char * str, ...)
 
     if (ctx->compressed)
     {
-	char zbuff[BUFSIZ];
+	unsigned char zbuff[BUFSIZ];
 
 	if  (ctx->zout.avail_in != 0)
 	{
@@ -634,7 +644,7 @@ static int refill_buffer(CvsServerCtx * ctx)
 		ctx->zin.avail_in = zlen;
 	    }
 	    
-	    ctx->zin.next_out = ctx->head;
+	    ctx->zin.next_out = (unsigned char *)ctx->head;
 	    ctx->zin.avail_out = len;
 	    
 	    /* FIXME: we don't always need Z_SYNC_FLUSH, do we? */
@@ -661,24 +671,29 @@ static int refill_buffer(CvsServerCtx * ctx)
     return len;
 }
 
-static int read_line(CvsServerCtx * ctx, char * p)
+static int read_line(CvsServerCtx * ctx, char * p, int maxlen)
 {
     int len = 0;
+
+    if (maxlen <= 0)
+	return -1;
+
     while (1)
     {
 	if (ctx->head == ctx->tail)
 	    if (refill_buffer(ctx) <= 0)
 		return -1;
 
-	*p = *ctx->head++;
-
-	if (*p == '\n')
+	/* break out without advancing head if buffer is exhausted */
+	if (maxlen == 1 || (*p = *ctx->head++) == '\n')
 	{
 	    *p = 0;
 	    break;
 	}
+
 	p++;
 	len++;
+	maxlen--;
     }
 
     return len;
@@ -689,7 +704,7 @@ static int read_response(CvsServerCtx * ctx, const char * str)
     /* FIXME: more than 1 char at a time */
     char resp[BUFSIZ];
 
-    if (read_line(ctx, resp) < 0)
+    if (read_line(ctx, resp, BUFSIZ) < 0)
 	return 0;
 
     debug(DEBUG_TCP, "response '%s' read", resp);
@@ -703,7 +718,7 @@ static void ctx_to_fp(CvsServerCtx * ctx, FILE * fp)
 
     while (1)
     {
-	read_line(ctx, line);
+	read_line(ctx, line, BUFSIZ);
 	debug(DEBUG_TCP, "ctx_to_fp: %s", line);
 	if (memcmp(line, "M ", 2) == 0)
 	{
@@ -712,7 +727,7 @@ static void ctx_to_fp(CvsServerCtx * ctx, FILE * fp)
 	}
 	else if (memcmp(line, "E ", 2) == 0)
 	{
-	    debug(DEBUG_APPMSG1, "%s", line + 2);
+	    debug(DEBUG_TCP, "%s", line + 2);
 	}
 	else if (strncmp(line, "ok", 2) == 0 || strncmp(line, "error", 5) == 0)
 	{
@@ -741,22 +756,9 @@ void cvs_rdiff(CvsServerCtx * ctx,
     ctx_to_fp(ctx, stdout);
 }
 
-void cvs_rupdate(CvsServerCtx * ctx, const char * rep, const char * file, const char * rev, int create, const char * opts)
+void cvs_rupdate(CvsServerCtx * ctx, const char * rep, const char * file, const char * rev, FILE *fp)
 {
-    FILE * fp;
-    char cmdbuff[BUFSIZ];
-    
-    snprintf(cmdbuff, BUFSIZ, "diff %s %s /dev/null %s | sed -e '%s s|^\\([+-][+-][+-]\\) -|\\1 %s/%s|g'",
-	     opts, create?"":"-", create?"-":"", create?"2":"1", rep, file);
-
-    debug(DEBUG_TCP, "cmdbuff: %s", cmdbuff);
-
-    if (!(fp = popen(cmdbuff, "w")))
-    {
-	debug(DEBUG_APPERROR, "cvs_direct: popen for diff failed: %s", cmdbuff);
-	exit(1);
-    }
-
+    send_string(ctx, "Argument -kk\n");
     send_string(ctx, "Argument -p\n");
     send_string(ctx, "Argument -r\n");
     send_string(ctx, "Argument %s\n", rev);
@@ -764,28 +766,26 @@ void cvs_rupdate(CvsServerCtx * ctx, const char * rep, const char * file, const
     send_string(ctx, "co\n");
 
     ctx_to_fp(ctx, fp);
-
-    pclose(fp);
 }
 
-static int parse_patch_arg(char * arg, char ** str)
+static bool parse_patch_arg(char * arg, char ** str)
 {
     char *tok, *tok2 = "";
     tok = strsep(str, " ");
     if (!tok)
-	return 0;
+	return false;
 
-    if (!*tok == '-')
+    if (*tok != '-')
     {
 	debug(DEBUG_APPERROR, "diff_opts parse error: no '-' starting argument: %s", *str);
-	return 0;
+	return false;
     }
     
     /* if it's not 'long format' argument, we can process it efficiently */
     if (tok[1] == '-')
     {
 	debug(DEBUG_APPERROR, "diff_opts parse_error: long format args not supported");
-	return 0;
+	return false;
     }
 
     /* see if command wants two args and they're separated by ' ' */
@@ -795,12 +795,12 @@ static int parse_patch_arg(char * arg, char ** str)
 	if (!tok2)
 	{
 	    debug(DEBUG_APPERROR, "diff_opts parse_error: argument %s requires two arguments", tok);
-	    return 0;
+	    return false;
 	}
     }
     
     snprintf(arg, 32, "%s%s", tok, tok2);
-    return 1;
+    return true;
 }
 
 void cvs_diff(CvsServerCtx * ctx, 
@@ -848,7 +848,7 @@ void cvs_diff(CvsServerCtx * ctx,
 /*
  * FIXME: the design of this sucks.  It was originally designed to fork a subprocess
  * which read the cvs response and send it back through a pipe the main process,
- * which fdopen(3)ed the other end, and juts used regular fgets.  This however
+ * which fdopen(3)ed the other end, and just used regular fgets.  This however
  * didn't work because the reads of compressed data in the child process altered
  * the compression state, and there was no way to resynchronize that state with
  * the parent process.  We could use threads...
@@ -879,7 +879,7 @@ char * cvs_rlog_fgets(char * buff, int buflen, CvsServerCtx * ctx)
     char lbuff[BUFSIZ];
     int len;
 
-    len = read_line(ctx, lbuff);
+    len = read_line(ctx, lbuff, BUFSIZ);
     debug(DEBUG_TCP, "cvs_direct: rlog: read %s", lbuff);
 
     if (memcmp(lbuff, "M ", 2) == 0)
@@ -890,9 +890,9 @@ char * cvs_rlog_fgets(char * buff, int buflen, CvsServerCtx * ctx)
     }
     else if (memcmp(lbuff, "E ", 2) == 0)
     {
-	debug(DEBUG_APPMSG1, "%s", lbuff + 2);
+	debug(DEBUG_TCP, "%s", lbuff + 2);
     }
-    else if (strcmp(lbuff, "ok") == 0 ||strcmp(lbuff, "error") == 0)
+    else if (strcmp(lbuff, "ok") == 0 || strncmp(lbuff, "error", 5) == 0)
     {
 	debug(DEBUG_TCP, "cvs_direct: rlog: got command completion");
 	return NULL;
@@ -905,18 +905,21 @@ void cvs_rlog_close(CvsServerCtx * ctx)
 {
 }
 
-void cvs_version(CvsServerCtx * ctx, char * client_version, char * server_version)
+void cvs_version(CvsServerCtx * ctx, char * client_version, char * server_version, int cvlen, int svlen)
 {
     char lbuff[BUFSIZ];
-    strcpy(client_version, "Client: Concurrent Versions System (CVS) 99.99.99 (client/server) cvs-direct");
+    strcpy_a(client_version, "Client: Concurrent Versions System (CVS) 99.99.99 (client/server) cvs-direct", cvlen);
     send_string(ctx, "version\n");
-    read_line(ctx, lbuff);
+    read_line(ctx, lbuff, BUFSIZ);
     if (memcmp(lbuff, "M ", 2) == 0)
-	sprintf(server_version, "Server: %s", lbuff + 2);
+	snprintf(server_version, svlen, "Server: %s", lbuff + 2);
     else
 	debug(DEBUG_APPERROR, "cvs_direct: didn't read version: %s", lbuff);
     
-    read_line(ctx, lbuff);
+    read_line(ctx, lbuff, BUFSIZ);
+    if (strstr(lbuff,"CVSACL")!=NULL) {
+	read_line(ctx, lbuff, BUFSIZ);
+    }
     if (strcmp(lbuff, "ok") != 0)
 	debug(DEBUG_APPERROR, "cvs_direct: protocol error reading version");
 
