@@ -7,9 +7,11 @@
 #define CVSPS_TYPES_H
 
 #include <time.h>
+#include <stdbool.h>
 
-#define LOG_STR_MAX 32768
+#define LOG_STR_MAX 65536
 #define AUTH_STR_MAX 64
+#define CID_STR_MAX 64
 #define REV_STR_MAX 64
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
@@ -21,11 +23,13 @@ typedef struct _PatchSetRange PatchSetRange;
 typedef struct _CvsFileRevision CvsFileRevision;
 typedef struct _GlobalSymbol GlobalSymbol;
 typedef struct _Tag Tag;
+typedef struct _TagName TagName;
+typedef struct _Branch Branch;
 
 struct _CvsFileRevision
 {
     char * rev;
-    int dead;
+    bool dead;
     CvsFile * file;
     char * branch;
     /*
@@ -33,7 +37,7 @@ struct _CvsFileRevision
      * revisions that don't exist. track 'confirmed' revisions
      * so as to not let them screw us up.
      */
-    int present;
+    bool present;
 
     /*
      * A revision can be part of many PatchSets because it may
@@ -70,7 +74,7 @@ struct _CvsFile
      * we don't have the branch info, so the CvsFileRevisions get created 
      * with the branch attribute NULL.  Later we need to resolve these.
      */
-    int have_branches;
+    bool have_branches;
 };
 
 struct _PatchSetMember
@@ -82,14 +86,14 @@ struct _PatchSetMember
     /*
      * bad_funk is only set w.r.t the -r tags
      */
-    int bad_funk;
+    bool bad_funk;
     struct list_head link;
 };
 
 /* 
  * these are bit flags for tag flags 
  * they apply to any patchset that
- * has an assoctiated tag
+ * has an associated tag
  */
 #define TAG_FUNKY   0x1
 #define TAG_INVALID 0x2
@@ -112,8 +116,8 @@ struct _PatchSet
     time_t max_date;
     char *descr;
     char *author;
-    char *tag;
-    int tag_flags;
+    char *commitid;
+    struct list_head tags;
     char *branch;
     char *ancestor_branch;
     struct list_head members;
@@ -123,7 +127,7 @@ struct _PatchSet
      * we want to ignore these.  fortunately, there's a way to detect them
      * without resorting to looking at the log message.
      */
-    int branch_add;
+    bool branch_add;
     /*
      * If the '-r' option specifies a funky tag, we will need to detect the
      * PatchSets that come chronologically before the tag, but are logically
@@ -131,9 +135,22 @@ struct _PatchSet
      */
     int funk_factor;
 
+    /*
+     * In fast-export mode, this is the mark generated for this patchset
+     * at the moment we try to emit it.  Has to be kept here because of
+     * the complications around first commits after branching.
+     */
+    int mark;              
+
+    /* 
+     * a list of 'Branch' objects that branch from here
+     */
+    struct list_head branches;
+
     /* for putting onto a list */
     struct list_head all_link;
     struct list_head collision_link;
+
 };
 
 struct _PatchSetRange
@@ -159,4 +176,19 @@ struct _Tag
     struct list_head rev_link;
 };
 
+struct _TagName
+{
+    char * name;
+    int flags;
+    struct list_head link;
+};
+
+struct _Branch
+{
+    char * name;
+    PatchSet * ps;
+    /* every patchset will have a list of branches that branch from there */
+    struct list_head link;
+};
+
 #endif /* CVSPS_TYPES_H */
