@@ -14,6 +14,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <regex.h>
+#include <stdbool.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -84,6 +85,7 @@ char *get_cvsps_dir()
     }
 
     /* Make sure the prefix directory exists */
+    /* coverity[toctou] */
     if (stat(prefix, &sbuf) < 0)
     {
 	int ret;
@@ -144,17 +146,22 @@ char *get_string(char const *str)
 static int get_int_substr(const char * str, const regmatch_t * p)
 {
     char buff[256];
+    if(p->rm_so == -1)
+	    return 0;
+    if(p->rm_eo - p->rm_so >= sizeof(buff))
+	    return 0;
     memcpy(buff, str + p->rm_so, p->rm_eo - p->rm_so);
     buff[p->rm_eo - p->rm_so] = 0;
     return atoi(buff);
 }
 
-static time_t mktime_utc(struct tm * tm)
+static time_t mktime_utc(struct tm * tm, const char* tzbuf)
 {
+    /* coverity[tainted_data] */
     char * old_tz = getenv("TZ");
     time_t ret;
 
-    setenv("TZ", "UTC", 1);
+    setenv("TZ", tzbuf, 1);
 
     tzset();
 	    
@@ -173,7 +180,7 @@ static time_t mktime_utc(struct tm * tm)
 void convert_date(time_t * t, const char * dte)
 {
     static regex_t date_re;
-    static int init_re;
+    static bool init_re;
 
 #define MAX_MATCH 16
     size_t nmatch = MAX_MATCH;
@@ -181,18 +188,20 @@ void convert_date(time_t * t, const char * dte)
 
     if (!init_re) 
     {
-	if (regcomp(&date_re, "([0-9]{4})[-/]([0-9]{2})[-/]([0-9]{2}) ([0-9]{2}):([0-9]{2}):([0-9]{2})", REG_EXTENDED)) 
+	if (regcomp(&date_re, "([0-9]{4})[-/]([0-9]{2})[-/]([0-9]{2})[ T]([0-9]{2}):([0-9]{2}):([0-9]{2})( [-+][0-9]{4})?", REG_EXTENDED)) 
 	{
 	    fprintf(stderr, "FATAL: date regex compilation error\n");
 	    exit(1);
 	}
-	init_re = 1;
+	init_re = true;
     }
     
     if (regexec(&date_re, dte, nmatch, match, 0) == 0)
     {
 	regmatch_t * pm = match;
 	struct tm tm = {0};
+	char tzbuf[32];
+	int offseth = 0, offsetm = 0;
 
 	/* first regmatch_t is match location of entire re */
 	pm++;
@@ -203,11 +212,17 @@ void convert_date(time_t * t, const char * dte)
 	tm.tm_hour = get_int_substr(dte, pm++);
 	tm.tm_min  = get_int_substr(dte, pm++);
 	tm.tm_sec  = get_int_substr(dte, pm++);
+	offseth    = -get_int_substr(dte, pm++);
+
+	offsetm= offseth%100;
+	if(offsetm<0) offsetm*=-1;
+	offseth/=100;
+	snprintf(tzbuf, sizeof(tzbuf), "UTC%+d:%d", offseth, offsetm);
 
 	tm.tm_year -= 1900;
 	tm.tm_mon--;
 
-	*t = mktime_utc(&tm);
+	*t = mktime_utc(&tm, tzbuf);
     }
     else
     {
@@ -236,34 +251,6 @@ void timing_stop(const char * msg)
 
 extern char ** environ;
 
-/* taken from the linux manual page for system */
-int my_system (const char *command) 
-{
-    int pid, status;
-    
-    if (command == 0)
-	return 1;
-    pid = fork();
-    if (pid == -1)
-	return -1;
-    if (pid == 0) {
-	char *argv[4];
-	argv[0] = "sh";
-	argv[1] = "-c";
-	argv[2] = (char*)command; /* discard const */
-	argv[3] = 0;
-	execve("/bin/sh", argv, environ);
-	exit(127);
-    }
-    do {
-	if (waitpid(pid, &status, 0) == -1) {
-	    if (errno != EINTR)
-		return -1;
-	} else
-	    return status;
-    } while(1);
-}
-
 int escape_filename(char * dst, int len, const char * src)
 {
     static char * naughty_chars = " \\\"'@<>=;|&()#$`?*[!:{";
@@ -289,3 +276,18 @@ int escape_filename(char * dst, int len, const char * src)
 
     return (*src == 0) ? 0 : -1;
 }
+
+void strcpy_a(char * dst, const char * src, int n)
+{
+    /* place a 'sentinel' char in final place in buffer 
+     * if strncpy expires the avail. space, this char.
+     * will no longer be \0
+     */
+    dst[n - 1] = 0;
+    strncpy(dst, src, n);
+
+    if (dst[n - 1] != 0) {
+	debug(DEBUG_APPERROR, "FATAL: internal buffer exhausted.");
+	exit(1);
+    }
+}
